package com.sun.xml.internal.ws.model.wsdl;

import com.sun.xml.internal.ws.resources.UtilMessages;
import com.sun.istack.internal.NotNull;
import com.sun.xml.internal.ws.api.model.wsdl.WSDLObject;
import javax.xml.ws.WebServiceException;
import org.xml.sax.Locator;
import javax.xml.namespace.QName;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.HashSet;
import javax.xml.stream.XMLStreamReader;
import java.util.List;
import com.sun.xml.internal.ws.api.model.wsdl.WSDLExtension;
import java.util.Set;
import com.sun.xml.internal.ws.api.model.wsdl.WSDLExtensible;

abstract class AbstractExtensibleImpl extends AbstractObjectImpl implements WSDLExtensible
{
    protected final Set<WSDLExtension> extensions;
    protected List<AbstractExtensibleImpl.UnknownWSDLExtension> notUnderstoodExtensions;
    
    protected AbstractExtensibleImpl(XMLStreamReader xsr) {
        super(xsr);
        this.extensions = new HashSet();
        this.notUnderstoodExtensions = new ArrayList();
    }
    
    protected AbstractExtensibleImpl(String systemId, int lineNumber) {
        super(systemId, lineNumber);
        this.extensions = new HashSet();
        this.notUnderstoodExtensions = new ArrayList();
    }
    
    public final Iterable<WSDLExtension> getExtensions() {
        return this.extensions;
    }
    
    public final <T extends WSDLExtension> Iterable<T> getExtensions(Class<T> type) {
        List<T> r = new ArrayList(this.extensions.size());
        for (WSDLExtension e : this.extensions) {
            if (type.isInstance(e)) {
                r.add(type.cast(e));
            }
            else {
                continue;
            }
        }
        return r;
    }
    
    public <T extends WSDLExtension> T getExtension(Class<T> type) {
        for (WSDLExtension e : this.extensions) {
            if (type.isInstance(e)) {
                return (T)type.cast(e);
            }
            continue;
        }
        return null;
    }
    
    public void addExtension(WSDLExtension ex) {
        if (ex == null) {
            throw new IllegalArgumentException();
        }
        this.extensions.add(ex);
        return;
    }
    
    public List<? extends AbstractExtensibleImpl.UnknownWSDLExtension> getNotUnderstoodExtensions() {
        return this.notUnderstoodExtensions;
    }
    
    public void addNotUnderstoodExtension(QName extnEl, Locator locator) {
        this.notUnderstoodExtensions.add(new AbstractExtensibleImpl.UnknownWSDLExtension(extnEl, locator));
    }
    
    public boolean areRequiredExtensionsUnderstood() {
        if (this.notUnderstoodExtensions.size() != 0) {
            StringBuilder buf = new StringBuilder("Unknown WSDL extensibility elements:");
            for (AbstractExtensibleImpl.UnknownWSDLExtension extn : this.notUnderstoodExtensions) {
                buf.append('\n').append(extn.toString());
            }
            throw new WebServiceException(buf.toString());
        }
        return true;
    }
    
    protected static class UnknownWSDLExtension implements WSDLExtension, WSDLObject
    {
        private final QName extnEl;
        private final Locator locator;
        
        public UnknownWSDLExtension(QName extnEl, Locator locator) {
            this.extnEl = extnEl;
            this.locator = locator;
        }
        
        public QName getName() {
            return this.extnEl;
        }
        
        @NotNull
        public Locator getLocation() {
            return this.locator;
        }
        
        @Override
        public String toString() {
            return this.extnEl + " " + UtilMessages.UTIL_LOCATION(Integer.valueOf(this.locator.getLineNumber()), this.locator.getSystemId());
        }
    }
}
